<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_IMAGEOBJECT_IMEXT_INCLUDED') ) {
	define ('_OPN_CLASS_IMAGEOBJECT_IMEXT_INCLUDED', 1);

	class imageobject_imext extends imageobject {

		/**
		 * Class constructor
		 * 		
		 * @param $directory
		 * @param $filename
		 */
		function /*<{4php5}>imageobject_imext!!__construct<{4php5/}>*//*<{delete}>*/imageobject_imext/*<{delete/}>*/ ($directory, $filename) {

			$this->PerformParams ($directory, $filename);
		}

		/**
		 * Set's the needed params
		 * 
		 * @param string $directory
		 * @param string $filename
		 */
		function PerformParams ($directory, $filename) {

			if (!extension_loaded ('imagick') ) {
				opnErrorHandler (E_WARNING, 'The imagick extension can not be found', __FILE__, __LINE__);
				die ();
			}
			$this->SetParams ($directory, $filename);
			$this->_truecolor = true;
			if (file_exists ($directory . $filename) ) {
				$this->_filesize = round (filesize ($this->_fullname)/1000);
				if ($this->_filesize>0) {
					$size = $this->GetImageInfo ($this->_fullname);
					if ($size) {
						$this->_imgRes = imagick_readimage ($this->_fullname);
						if (imagick_iserror ($this->_imgRes) ) {
							$this->_setError ('readimage');
						}
					}
					$this->_width = $size[0];
					$this->_height = $size[1];
					$this->_string = $size[3];
				}
			}
			// if
		}

		/**
		* Crop an image
		*
		* @param $clipval
		* @access public
		*/

		function cropImage (&$clipval) {

			global $opnConfig;

			$cliparray = split (",", $clipval);
			$clip_top = $cliparray[0];
			$clip_right = $cliparray[1];
			$clip_bottom = $cliparray[2];
			$clip_left = $cliparray[3];
			$new_w = $clip_right- $clip_left;
			$new_h = $clip_bottom- $clip_top;
			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			if (!imagick_crop ($this->_imgRes, $clip_left, $clip_top, $new_w, $new_h) ) {
				$this->_setError ('crop');
				return false;
			}
			if (!imagick_writeimage ($this->_imgRes, $this->_fullname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			imagick_destroyhandle ($this->_imgRes);
			// Call the constructor again to repopulate the dimensions etc
			$this->PerformParams ($this->_directory, $this->_filename);

		}

		/**
		* Resize an image
		*
		* @param int	$new_w	new width
		* @param int	$new_h	new height
		* @access public
		*/

		function resizeImage ($new_w = 0, $new_h = 0) {

			global $opnConfig;

			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			if (!imagick_resize ($this->_imgRes, $new_w, $new_h, IMAGICK_FILTER_UNKNOWN, 1) ) {
				$this->_setError ('resize');
				return false;
			}
			if (!imagick_writeimage ($this->_imgRes, $this->_fullname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			imagick_destroyhandle ($this->_imgRes);
			// Call the constructor again to repopulate the dimensions etc
			$this->PerformParams ($this->_directory, $this->_filename);

		}

		/**
		* Rotate an image
		*
		* @param $angle
		* @access public
		*/

		function rotateImage (&$angle) {

			global $opnConfig;

			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			if (!imagick_rotate ($this->_imgRes, $angle) ) {
				$this->_setError ('resize');
				return false;
			}
			if (!imagick_writeimage ($this->_imgRes, $this->_fullname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			imagick_destroyhandle ($this->_imgRes);
			// Call the constructor again to repopulate the dimensions etc
			$this->PerformParams ($this->_directory, $this->_filename);

		}

		/**
		 * Mirror an image horizontal, vertical or both
		 * 		
		 * @param $type
		 */
		function mirrorImage ($type) {

			global $opnConfig;

			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			switch ($type) {
				case _MIRRORFLOP:
					if (!imagick_flop ($this->_imgRes) ) {
						$this->_setError ('flop');
						return false;
					}
					break;
				case _MIRRORFLIP:
					if (!imagick_flip ($this->_imgRes) ) {
						$this->_setError ('flip');
						return false;
					}
					break;
				case _MIRROR_FLIPFLOP:
					if (!imagick_flip ($this->_imgRes) ) {
						$this->_setError ('flip');
						return false;
					}
					if (!imagick_flop ($this->_imgRes) ) {
						$this->_setError ('flop');
						return false;
					}
					break;
			} /* switch */
			if (!imagick_writeimage ($this->_imgRes, $this->_fullname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			imagick_destroyhandle ($this->_imgRes);
			// Call the constructor again to repopulate the dimensions etc
			$this->PerformParams ($this->_directory, $this->_filename);

		}

		/**
		* Get the imageinfo array
		*
		* @param $filename
		* @access public
		* @return mixed bool or array
		*/

		function GetImageInfo ($filename) {
			if (!file_exists ($filename) ) {
				return false;
			}
			$imgRes = imagick_readimage ($filename);
			if (imagick_iserror ($imgRes) ) {
				$this->_setError ('readimage');
				return false;
			}
			if (imagick_getlistsize ($imgRes) > 1) {
				$imginfo = $this->_get_animated_dimension ($imgRes);
				imagick_destroyhandle ($imgRes);
			} else {
				$imginfo[0] = imagick_getwidth ($imgRes);
				if (imagick_iserror ($imgRes) ) {
					$this->_setError ('gewidth');
					return false;
				}
				$imginfo[1] = imagick_getheight ($imgRes);
				if (imagick_iserror ($imgRes) ) {
					$this->_setError ('getheight');
					return false;
				}
				$mime = imagick_getmimetype ($imgRes);
				if (imagick_iserror ($imgRes) ) {
					$this->_setError ('getmimetype');
					return false;
				}
				imagick_destroyhandle ($imgRes);
				$mime = explode ('/', $mime);
				if ($mime[1] == 'jpeg') {
					$imginfo[2] = IMAGETYPE_JPEG;
				} else {
					$imginfo[2] = $mime[1];
				}
				if ($mime[1] == 'svg') {
					$imginfo[0] += 1;
					$imginfo[1] += 1;
				}
				$imginfo[3] = 'height="' . $imginfo[1] . '" width="' . $imginfo[0] . '"';
			}
			return $imginfo;

		}

		/**
		* Creates a thumbnail for the image
		*
		* @param $filename
		* @param $thumbname
		* @param $width
		* @param $height
		* @param $docheck
		* @return bool
		*/
		function CreateThumbnail ($filename, $thumbname, $width, $height, $docheck = false) {

			$this->_imgRes = imagick_readimage ($filename);
			if (imagick_iserror ($this->_imgRes) ) {
				return false;
			}
			$prop = '';
			$commando = $width . 'x' . $height;
			if (!$this->_prop_resize) {
				$prop = '!';
			} else {
				if ($docheck) {
					$imginfo = $this->GetImageInfo ($filename);
					$w = $imginfo[0];
					$h = $imginfo[1];
					if ($w < $h) {
						$commando = $width . 'x';
					} elseif ($h < $w) {
						$commando = 'x' . $height;
					}
				}
			}
			$prop .= '>';
			$commando = $commando . $prop;
			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			if (!$docheck) {
				$commando = $prop;
			}
			if (!imagick_resize ($this->_imgRes, $width, $height, IMAGICK_FILTER_UNKNOWN, 0, $commando) ) {
				$this->_setError ('resize');
				return false;
			}
			if (!imagick_writeimage ($this->_imgRes, $thumbname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			$scenes = imagick_getlistsize ($this->_imgRes);
			if ($scenes > 1) {
				$file =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opnFile ();
				if (file_exists ($thumbname)) {
					$file->delete_File ($thumbname);
				}
				$scene0 = $this->_get_scene_filename ($thumbname, 0);
				$file->rename_file($scene0, $thumbname, '0666');
				unset($file);
			}
			imagick_destroyhandle ($this->_imgRes);
			return true;

		}

		/**
		* Converts an image to a another format (normaly JPG).
		*
		* @param $filename
		* @param $convertname
		* @return bool
		*/

		function ConvertImage ($filename, $convertname) {

			$this->_imgRes = imagick_readimage ($filename);
			if (imagick_iserror ($this->_imgRes) ) {
				$this->_setError ('readimage');
				return false;
			}
			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			if (!imagick_writeimage ($this->_imgRes, $convertname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			imagick_destroyhandle ($this->_imgRes);
			return true;

		}

		/**
		 * Size an image
		 * 		
		 * @param $width
		 * @param $height
		 * @return boolean
		 */
		function SizeImage ($width, $height) {


			global $opnConfig;

			imagick_set_image_quality ($this->_imgRes, $this->_quality);
			if (!imagick_resize ($this->_imgRes, $width, $height, IMAGICK_FILTER_UNKNOWN, 1) ) {
				$this->_setError ('resize');
				return false;
			}
			if (!imagick_writeimage ($this->_imgRes, $this->_fullname) ) {
				$this->_setError ('writeimage');
				return false;
			}
			imagick_destroyhandle ($this->_imgRes);
			return true;
		}

		// private functions

		function _setError ($function) {

			$reason = imagick_failedreason ($this->_imgRes);
			$description = imagick_faileddescription ($this->_imgRes);
			$this->_message .= $function . ' failed<br />Reason: ' . $reason . '<br />Description: ' . $description . '<br />';

		}

		/**
		 * Returns the max. width and height for a mediafile with more than one image
		 * 		
		 * @param $imgRes
		 * @return mixed boolean or array
		 */
		function _get_animated_dimension ($imgRes) {

			global $opnConfig;

			$imginfo = array ();
			$imginfo[0] = 0;
			$imginfo[1] = 0;
			$mime = imagick_getmimetype ($imgRes);
			if (imagick_iserror ($imgRes) ) {
				$this->_setError ('getmimetype');
				return false;
			}
			if ($mime[1] == 'jpeg') {
				$imginfo[2] = IMAGETYPE_JPEG;
			} else {
				$imginfo[2] = $mime[1];
			}
			$scenes = imagick_getlistsize ($imgRes);
			for ($i = 0; $i < $scenes; $i++) {
				$imginfo[0] = max ($imginfo[0], imagick_getwidth ($imgRes));
				$imginfo[1] = max ($imginfo[1], imagick_getheight ($imgRes));
				imagick_next($imgRes);
			}
			$imginfo[3] = 'height="' . $imginfo[1] . '" width="' . $imginfo[0] . '"';
			return $imginfo;
		}

	}
}

?>